@@ -61,7 +61,7 @@ if (is.null(temperatures)) {
 if (!is.null(parameters)) {
 	newx <- .SSM(temperatures, parameters)[[1]]*1E5
 } else 
-  if (inherits(nests, "NestsResult"))  #(class(nests)=="NestsResult") 	
+  if (inherits(nests, "NestsResult")) 	
     newx <- .SSM(temperatures, nests$par)[[1]]*1E5
 
 
